/* automatically generated from pulseaudio-dialog.glade */
#ifdef __SUNPRO_C
#pragma align 4 (pulseaudio_dialog_ui)
#endif
#ifdef __GNUC__
static const char pulseaudio_dialog_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char pulseaudio_dialog_ui[] =
#endif
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?><interface><requires lib=\"gt"
  "k+\" version=\"3.20\"/><requires lib=\"libxfce4ui-2\" version=\"4.11\"/"
  "><object class=\"GtkImage\" id=\"image1\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"icon_name\">xfce4-pulseaudio-plugin</property></object><object clas"
  "s=\"GtkImage\" id=\"image2\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"icon_name"
  "\">edit-clear</property></object><object class=\"GtkListStore\" id=\"li"
  "ststore1\"><columns><column type=\"gchararray\"/><column type=\"gcharar"
  "ray\"/><column type=\"gchararray\"/><column type=\"gboolean\"/></column"
  "s></object><object class=\"XfceTitledDialog\" id=\"dialog\"><property n"
  "ame=\"can_focus\">False</property><property name=\"title\" translatable"
  "=\"yes\">PulseAudio Panel Plugin</property><property name=\"default_wid"
  "th\">425</property><property name=\"default_height\">525</property><pro"
  "perty name=\"icon_name\">gtk-properties</property><property name=\"type"
  "_hint\">normal</property><child internal-child=\"vbox\"><object class=\""
  "GtkBox\" id=\"dialog-vbox2\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"orientati"
  "on\">vertical</property><property name=\"spacing\">2</property><child i"
  "nternal-child=\"action_area\"><object class=\"GtkButtonBox\" id=\"dialo"
  "g-action_area2\"><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">False</property><property name=\"layout_style\">end</p"
  "roperty><child><object class=\"GtkButton\" id=\"close-button\"><propert"
  "y name=\"label\">gtk-close</property><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">True</property><property name=\"re"
  "ceives_default\">True</property><property name=\"use_stock\">True</prop"
  "erty></object><packing><property name=\"expand\">False</property><prope"
  "rty name=\"fill\">False</property><property name=\"position\">0</proper"
  "ty></packing></child><child><object class=\"GtkButton\" id=\"help-butto"
  "n\"><property name=\"label\">gtk-help</property><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">True</property><propert"
  "y name=\"receives_default\">True</property><property name=\"use_stock\""
  ">True</property></object><packing><property name=\"expand\">False</prop"
  "erty><property name=\"fill\">False</property><property name=\"position\""
  ">1</property><property name=\"secondary\">True</property></packing></ch"
  "ild></object><packing><property name=\"expand\">False</property><proper"
  "ty name=\"fill\">True</property><property name=\"pack_type\">end</prope"
  "rty><property name=\"position\">0</property></packing></child><child><o"
  "bject class=\"GtkNotebook\" id=\"sections\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">True</property><property na"
  "me=\"border_width\">6</property><child><object class=\"GtkBox\" id=\"se"
  "ction_general_content\"><property name=\"visible\">True</property><prop"
  "erty name=\"can_focus\">False</property><property name=\"border_width\""
  ">12</property><property name=\"orientation\">vertical</property><proper"
  "ty name=\"spacing\">12</property><child><object class=\"GtkFrame\" id=\""
  "frame1\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">False</property><property name=\"label_xalign\">0</property><p"
  "roperty name=\"shadow_type\">none</property><child><object class=\"GtkA"
  "lignment\" id=\"alignment1\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"top_paddi"
  "ng\">6</property><property name=\"left_padding\">12</property><property"
  " name=\"right_padding\">12</property><child><object class=\"GtkBox\" id"
  "=\"vbox2\"><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">False</property><property name=\"orientation\">vertical</pro"
  "perty><property name=\"spacing\">6</property><child><object class=\"Gtk"
  "CheckButton\" id=\"checkbutton-keyboard-shortcuts\"><property name=\"la"
  "bel\" translatable=\"yes\">Enable keyboard _shortcuts for volume contro"
  "l</property><property name=\"visible\">True</property><property name=\""
  "can_focus\">True</property><property name=\"receives_default\">False</p"
  "roperty><property name=\"tooltip_text\" translatable=\"yes\">Enables vo"
  "lume control using multimedia keys. Make sure no other application that"
  " listens to these keys (e.g. xfce4-volumed) is running in the backgroun"
  "d.</property><property name=\"halign\">start</property><property name=\""
  "use_underline\">True</property><property name=\"draw_indicator\">True</"
  "property></object><packing><property name=\"expand\">True</property><pr"
  "operty name=\"fill\">True</property><property name=\"position\">0</prop"
  "erty></packing></child><child><object class=\"GtkCheckButton\" id=\"che"
  "ckbutton-show-notifications\"><property name=\"label\" translatable=\"y"
  "es\">Show _notifications when volume changes</property><property name=\""
  "visible\">True</property><property name=\"can_focus\">True</property><p"
  "roperty name=\"receives_default\">False</property><property name=\"tool"
  "tip_text\" translatable=\"yes\">Enables on-screen volume notifications."
  "</property><property name=\"halign\">start</property><property name=\"u"
  "se_underline\">True</property><property name=\"draw_indicator\">True</p"
  "roperty></object><packing><property name=\"expand\">True</property><pro"
  "perty name=\"fill\">True</property><property name=\"position\">1</prope"
  "rty></packing></child></object></child></object></child><child type=\"l"
  "abel\"><object class=\"GtkLabel\" id=\"label1\"><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">False</property><proper"
  "ty name=\"label\" translatable=\"yes\">Behaviour</property><attributes>"
  "<attribute name=\"weight\" value=\"bold\"/></attributes></object></chil"
  "d></object><packing><property name=\"expand\">False</property><property"
  " name=\"fill\">True</property><property name=\"position\">0</property><"
  "/packing></child><child><object class=\"GtkFrame\" id=\"frame2\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"label_xalign\">0</property><property name=\""
  "shadow_type\">none</property><child><object class=\"GtkAlignment\" id=\""
  "alignment2\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"top_padding\">6</property>"
  "<property name=\"left_padding\">12</property><property name=\"right_pad"
  "ding\">12</property><child><object class=\"GtkGrid\"><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">False</property><p"
  "roperty name=\"row_spacing\">6</property><property name=\"column_spacin"
  "g\">12</property><child><object class=\"GtkLabel\" id=\"label2\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"label\" translatable=\"yes\">Audio _Mixer</p"
  "roperty><property name=\"use_underline\">True</property></object><packi"
  "ng><property name=\"left_attach\">0</property><property name=\"top_atta"
  "ch\">0</property></packing></child><child><object class=\"GtkEntry\" id"
  "=\"entry-mixer-command\"><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">True</property><property name=\"tooltip_text\""
  " translatable=\"yes\">Audio mixer command that can be executed from the"
  " context menu, e.g. \"pavucontrol\", \"unity-control-center sound\".</p"
  "roperty><property name=\"hexpand\">True</property><property name=\"invi"
  "sible_char\">\342\200\242</property><property name=\"primary_icon_activ"
  "atable\">False</property><property name=\"secondary_icon_activatable\">"
  "False</property></object><packing><property name=\"left_attach\">1</pro"
  "perty><property name=\"top_attach\">0</property></packing></child><chil"
  "d><object class=\"GtkButton\" id=\"button-run-mixer\"><property name=\""
  "label\" translatable=\"yes\">_Run Audio Mixer...</property><property na"
  "me=\"visible\">True</property><property name=\"can_focus\">True</proper"
  "ty><property name=\"receives_default\">True</property><property name=\""
  "halign\">start</property><property name=\"image\">image1</property><pro"
  "perty name=\"use_underline\">True</property></object><packing><property"
  " name=\"left_attach\">0</property><property name=\"top_attach\">1</prop"
  "erty><property name=\"width\">2</property></packing></child></object></"
  "child></object></child><child type=\"label\"><object class=\"GtkLabel\""
  " id=\"label3\"><property name=\"visible\">True</property><property name"
  "=\"can_focus\">False</property><property name=\"label\" translatable=\""
  "yes\">Sound Settings</property><attributes><attribute name=\"weight\" v"
  "alue=\"bold\"/></attributes></object></child></object><packing><propert"
  "y name=\"expand\">False</property><property name=\"fill\">True</propert"
  "y><property name=\"position\">1</property></packing></child></object></"
  "child><child type=\"tab\"><object class=\"GtkLabel\" id=\"section_gener"
  "al\"><property name=\"visible\">True</property><property name=\"can_foc"
  "us\">False</property><property name=\"label\" translatable=\"yes\">Gene"
  "ral</property></object><packing><property name=\"tab_fill\">False</prop"
  "erty></packing></child><child><object class=\"GtkBox\" id=\"section_mp_"
  "content\"><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">False</property><property name=\"border_width\">10</property>"
  "<property name=\"orientation\">vertical</property><property name=\"spac"
  "ing\">12</property><child><object class=\"GtkBox\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">False</property><pro"
  "perty name=\"margin_left\">2</property><property name=\"margin_right\">"
  "2</property><property name=\"orientation\">vertical</property><child><o"
  "bject class=\"GtkBox\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><child><object class=\"GtkLabel\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"label\" translatable=\"yes\">Control "
  "Playback of Media Players</property><attributes><attribute name=\"weigh"
  "t\" value=\"bold\"/></attributes></object><packing><property name=\"exp"
  "and\">False</property><property name=\"fill\">True</property><property "
  "name=\"position\">0</property></packing></child><child><object class=\""
  "GtkSwitch\" id=\"switch-mpris-support\"><property name=\"visible\">True"
  "</property><property name=\"can_focus\">True</property></object><packin"
  "g><property name=\"expand\">False</property><property name=\"fill\">Tru"
  "e</property><property name=\"pack_type\">end</property><property name=\""
  "position\">1</property></packing></child></object><packing><property na"
  "me=\"expand\">False</property><property name=\"fill\">True</property><p"
  "roperty name=\"position\">0</property></packing></child><child><object "
  "class=\"GtkAlignment\" id=\"section_mp_content_1\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">False</property><pro"
  "perty name=\"top_padding\">3</property><property name=\"left_padding\">"
  "12</property><property name=\"right_padding\">12</property><child><obje"
  "ct class=\"GtkBox\"><property name=\"visible\">True</property><property"
  " name=\"can_focus\">False</property><property name=\"orientation\">vert"
  "ical</property><property name=\"spacing\">6</property><child><object cl"
  "ass=\"GtkCheckButton\" id=\"checkbutton-multimedia-keys\"><property nam"
  "e=\"label\" translatable=\"yes\">Enable multimedia keys for playback co"
  "ntrol</property><property name=\"visible\">True</property><property nam"
  "e=\"can_focus\">True</property><property name=\"receives_default\">Fals"
  "e</property><property name=\"halign\">start</property><property name=\""
  "draw_indicator\">True</property></object><packing><property name=\"expa"
  "nd\">False</property><property name=\"fill\">True</property><property n"
  "ame=\"position\">0</property></packing></child><child><object class=\"G"
  "tkCheckButton\" id=\"checkbutton-wnck\"><property name=\"label\" transl"
  "atable=\"yes\">Enable experimental window focus support</property><prop"
  "erty name=\"visible\">True</property><property name=\"can_focus\">True<"
  "/property><property name=\"receives_default\">False</property><property"
  " name=\"halign\">start</property><property name=\"draw_indicator\">True"
  "</property></object><packing><property name=\"expand\">False</property>"
  "<property name=\"fill\">True</property><property name=\"position\">1</p"
  "roperty></packing></child></object></child></object><packing><property "
  "name=\"expand\">False</property><property name=\"fill\">True</property>"
  "<property name=\"position\">1</property></packing></child></object><pac"
  "king><property name=\"expand\">False</property><property name=\"fill\">"
  "True</property><property name=\"position\">0</property></packing></chil"
  "d><child><object class=\"GtkFrame\" id=\"section_mp_content_2\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">False</"
  "property><property name=\"margin_left\">2</property><property name=\"ma"
  "rgin_right\">2</property><property name=\"margin_bottom\">12</property>"
  "<property name=\"label_xalign\">0</property><property name=\"shadow_typ"
  "e\">none</property><child><object class=\"GtkAlignment\"><property name"
  "=\"visible\">True</property><property name=\"can_focus\">False</propert"
  "y><property name=\"top_padding\">6</property><property name=\"left_padd"
  "ing\">12</property><property name=\"right_padding\">12</property><child"
  "><object class=\"GtkBox\"><property name=\"visible\">True</property><pr"
  "operty name=\"can_focus\">False</property><property name=\"orientation\""
  ">vertical</property><property name=\"spacing\">6</property><child><obje"
  "ct class=\"GtkScrolledWindow\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">True</property><property name=\"shadow_t"
  "ype\">in</property><child><object class=\"GtkTreeView\" id=\"player_tre"
  "e_view\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">True</property><property name=\"model\">liststore1</property><"
  "property name=\"headers_clickable\">False</property><property name=\"en"
  "able_search\">False</property><property name=\"show_expanders\">False</"
  "property><child internal-child=\"selection\"><object class=\"GtkTreeSel"
  "ection\"/></child><child><object class=\"GtkTreeViewColumn\" id=\"col_i"
  "con\"><child><object class=\"GtkCellRendererPixbuf\" id=\"col_icon_rend"
  "erer\"/><attributes><attribute name=\"icon-name\">0</attribute></attrib"
  "utes></child></object></child><child><object class=\"GtkTreeViewColumn\""
  " id=\"col_label\"><property name=\"title\" translatable=\"yes\">Title</"
  "property><property name=\"expand\">True</property><child><object class="
  "\"GtkCellRendererText\" id=\"col_label_renderer\"/><attributes><attribu"
  "te name=\"text\">2</attribute></attributes></child></object></child><ch"
  "ild><object class=\"GtkTreeViewColumn\" id=\"col_hidden\"><property nam"
  "e=\"title\" translatable=\"yes\">Hidden</property><child><object class="
  "\"GtkCellRendererToggle\" id=\"col_hidden_renderer\"/><attributes><attr"
  "ibute name=\"active\">3</attribute></attributes></child></object></chil"
  "d></object></child></object><packing><property name=\"expand\">True</pr"
  "operty><property name=\"fill\">True</property><property name=\"position"
  "\">0</property></packing></child><child><object class=\"GtkButton\" id="
  "\"clear_players\"><property name=\"label\" translatable=\"yes\">Clear K"
  "nown Items</property><property name=\"visible\">True</property><propert"
  "y name=\"can_focus\">True</property><property name=\"receives_default\""
  ">True</property><property name=\"halign\">start</property><property nam"
  "e=\"image\">image2</property></object><packing><property name=\"expand\""
  ">False</property><property name=\"fill\">False</property><property name"
  "=\"position\">1</property></packing></child><child><object class=\"GtkR"
  "evealer\" id=\"restart_revealer\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">False</property><child><object class="
  "\"GtkLabel\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"halign\">start</property><"
  "property name=\"label\" translatable=\"yes\">Please restart your panel "
  "for additional players to be displayed.</property><property name=\"wrap"
  "\">True</property><property name=\"xalign\">0</property><attributes><at"
  "tribute name=\"style\" value=\"italic\"/></attributes></object></child>"
  "</object><packing><property name=\"expand\">False</property><property n"
  "ame=\"fill\">True</property><property name=\"position\">2</property></p"
  "acking></child></object></child></object></child><child type=\"label\">"
  "<object class=\"GtkLabel\"><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">False</property><property name=\"label\" tra"
  "nslatable=\"yes\">Known Media Players</property><attributes><attribute "
  "name=\"weight\" value=\"bold\"/></attributes></object></child></object>"
  "<packing><property name=\"expand\">True</property><property name=\"fill"
  "\">True</property><property name=\"position\">1</property></packing></c"
  "hild></object><packing><property name=\"position\">1</property></packin"
  "g></child><child type=\"tab\"><object class=\"GtkLabel\" id=\"section_m"
  "p\"><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">False</property><property name=\"label\" translatable=\"yes\">Media"
  " Players</property></object><packing><property name=\"position\">1</pro"
  "perty><property name=\"tab_fill\">False</property></packing></child><ch"
  "ild><placeholder/></child><child type=\"tab\"><placeholder/></child></o"
  "bject><packing><property name=\"expand\">True</property><property name="
  "\"fill\">True</property><property name=\"position\">1</property></packi"
  "ng></child></object></child><action-widgets><action-widget response=\"0"
  "\">close-button</action-widget><action-widget response=\"0\">help-butto"
  "n</action-widget></action-widgets></object></interface>"
};

static const unsigned pulseaudio_dialog_ui_length = 16780u;

